#include "SDLConfig.h"

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#define SDLCFG_MAX_LINE 1024

Config::Config(char* _path, Platform* platform):
path(_path), fullscreen(true), verbose(false), effects(true), showConfig(true), music(true), soundEffects(true), mixingFrequency(44100),
fontPaths(NULL), mixOptions(NULL), module(NULL), article(NULL), externalLink(NULL),
configMap(NULL)
{
	fontPaths	= Vector_construct();
	mixOptions	= Vector_construct();
	configMap	= Map_createMap(10);
	
	char* line = new char[SDLCFG_MAX_LINE];
	
	/* Read configuration from file */
	FILE* file = fopen(path, "r");
	if (file != NULL)
	{
		while ( fgets(line, SDLCFG_MAX_LINE, file) != NULL )
		{
			/* Ignore all comments */
			if (line[0] != '#')
			{
				char* divider = strchr(line, '=');
				
				if (divider != NULL)
				{
					/* Split the line in two strings */
					*divider = 0;
					
					/* Copy the strings, and store them */
					char* key = strdup(line);
					char* value = strdup( divider+1);
					/* Delete the ending newline character */
					value[ strlen(value)-1] = 0;
					
					Map_put(configMap, key, value);
				}
				else
				{
					printf("Malformed line in config. No '=' character, and not a comment: %s\n", line);
				}
			}
		}
		
		readBool(SDLCFG_FULLSCREEN, &fullscreen);
		readBool(SDLCFG_VERBOSE, &verbose);
		readBool(SDLCFG_EFFECTS, &effects);
		readBool(SDLCFG_SHOW_CONFIG, &showConfig);
		readBool(SDLCFG_MUSIC, &music);
		readBool(SDLCFG_SOUND_EFFECTS, &soundEffects);
		readUint(SDLCFG_MIXING_FREQUENCY, &mixingFrequency);
		readStringCollection(fontPaths, SDLCFG_FONT_PATHS);
		readStringCollection(mixOptions, SDLCFG_MIX_OPTIONS);
		readString(SDLCFG_EXTERNAL_LINK, &externalLink);
		
		if (Vector_getSize(fontPaths) == 0)
		{
			char* fontsDir = new char[1024];
			snprintf(fontsDir, 1024, "%s%s%s", platform->getCurrentDir(), "fonts", platform->getDirSeparator() );
			
			/* Create what is needed for a minimal configuration */
			Vector_add(fontPaths, (void*)strdup( platform->getCurrentDir()));
			Vector_add(fontPaths, (void*)fontsDir);
		}
		if (Vector_getSize(mixOptions) == 0)
		{
			Vector_add(mixOptions, (void*)strdup("44100"));
			Vector_add(mixOptions, (void*)strdup("22050"));
			Vector_add(mixOptions, (void*)strdup("11025"));
		}
		fclose(file);
	}
	else
	{
		printf("Could not read config file at path : %s!\n", path);
		
		char* fontsDir = new char[1024];
		snprintf(fontsDir, 1024, "%s%s%s", platform->getCurrentDir(), "fonts", platform->getDirSeparator() );
		
		/* Create what is needed for a minimal configuration */
		Vector_add(fontPaths, strdup(platform->getDefaultFontPath()) );
		Vector_add(mixOptions, (void*)strdup("44100"));
		Vector_add(mixOptions, (void*)strdup("22050"));
		Vector_add(mixOptions, (void*)strdup("11025"));
	}
	delete[] line;	
}

bool Config::getFullscreen()
{
	return fullscreen;
}

bool Config::getVerbose()
{
	return verbose;
}

bool Config::getEffects()
{
	return effects;
}

bool Config::getShowConfig()
{
	return showConfig;
}

bool Config::getMusic()
{
	return music;
}
bool Config::getSoundEffects()
{
	return soundEffects;
}

bool* Config::getMusicPtr()
{
	return &music;
}
bool* Config::getSoundEffectsPtr()
{
	return &soundEffects;
}


bool* Config::getFullscreenPtr()
{
	return &fullscreen;
}

bool* Config::getVerbosePtr()
{
	return &verbose;
}

bool* Config::getEffectsPtr()
{
	return &effects;
}

bool* Config::getShowConfigPtr()
{
	return &showConfig;
}


Uint32 Config::getMixingFrequency()
{
	return mixingFrequency;
}

Vector* Config::getFontPaths()
{
	return fontPaths;
}

void Config::setMusic(bool _music)
{
	music = _music;
}
void Config::setSoundEffects(bool _soundEffects)
{
	soundEffects = _soundEffects;
}

void Config::setFullscreen(bool _fullscreen)
{
	fullscreen = _fullscreen;
}

void Config::setVerbose(bool _verbose)
{
	verbose = _verbose;
}

void Config::setEffects(bool _effects)
{
	effects = _effects;
}

void Config::setShowConfig(bool _showConfig)
{
	showConfig = _showConfig;
}

void Config::setMixingFrequency(Uint32 _mixingFrequency)
{
	mixingFrequency = _mixingFrequency;
}

bool Config::readBool(char* keyword, bool* value)
{
	char* entry = (char*) Map_get(configMap, keyword);
	if (entry != NULL)
	{
		if (strcasecmp(entry, "true") == 0)
		{
			*value = true;
			return true;
		}
		else if (strcasecmp(entry, "false") == 0)
		{
			*value = false;
			return true;
		}
		printf("Error in config file, keyword %s should have a boolean value, but has: %s.\n", keyword, entry);
	}	
	printf("No entry for keyword %s in config file, should be boolean.\n", keyword);
	
	return false;
}

bool Config::readString(char* keyword, char** value)
{
	char* entry = (char*) Map_get(configMap, keyword);
	
	if (entry != NULL)
	{
		*value = entry;
		
		return true;
	}
	printf("No entry for keyword %s in config file, should be a string.\n", keyword);

	return false;
}

bool Config::readUint(char* keyword, Uint32* value)
{
	char* entry = (char*) Map_get(configMap, keyword);
	
	if (entry != NULL)
	{
		char* end = NULL;
		long convert = strtol(entry, &end, 10);

		if (entry != end)
		{
			*value = (Uint32) convert;
			return true;
		}
		printf("Error in config file, keyword %s should have a positive integer value, but has: %s.\n", keyword, entry);
	}
	printf("No entry for keyword %s in config file, should be a positive integer.\n", keyword);
	
	return false;
}

bool Config::readStringCollection(Vector* vector, char* keyword)
{
	char* entry = (char*) Map_get(configMap, keyword);
	
	if (entry != NULL)
	{
		char* next = strchr(entry, ';');

		while (next != NULL)
		{
			/* Dirty little trick to ease the copying */
			*next = 0;
			char* newString = strdup(entry);
			*next = ';';
			
			Vector_add(vector, newString);
			
			entry = next+1;
			
			next = strchr(entry, ';');
		}
		Vector_add(vector, strdup(entry));
		return true;
	}
	printf("No entry for keyword %s in config file, should be a collection of strings separated by ';' characters.\n", keyword);
	
	return false;
}


void Config::writeBool(FILE* file, char* keyword, bool value)
{
	fputs(keyword, file);
	fputc('=', file);
	if (value)
		fputs("true", file);
	else
		fputs("false", file);
	fputc('\n', file);
}

void Config::writeInt(FILE* file, char* keyword, int value)
{
	fprintf(file, "%s=%i\n", keyword, value);
}

void Config::writeString(FILE* file, char* keyword, char* value)
{
	if (value == NULL)
		value = "";
	fprintf(file, "%s=%s\n", keyword, value);
}

void Config::save()
{
	FILE* file = fopen(path, "w");

	if (file != NULL)
	{
		char* writePaths 	= vectorToString(fontPaths);
		char* writeMix		= vectorToString(mixOptions);
		
		writeString(file, SDLCFG_FONT_PATHS, writePaths);
		writeString(file, SDLCFG_MIX_OPTIONS, writeMix);
		writeBool(file, SDLCFG_SHOW_CONFIG, getShowConfig());
		writeBool(file, SDLCFG_EFFECTS, getEffects());
		writeInt(file, SDLCFG_MIXING_FREQUENCY, getMixingFrequency());
		writeBool(file, SDLCFG_FULLSCREEN, getFullscreen());
		writeBool(file, SDLCFG_VERBOSE, getVerbose());
		writeBool(file, SDLCFG_MUSIC, getMusic());
		writeBool(file, SDLCFG_SOUND_EFFECTS, getSoundEffects());
		writeString(file, SDLCFG_EXTERNAL_LINK, externalLink);
		fclose(file);
	}
	else
	{
		printf("Could not save config to path: %s\n", path);
	}
}

char* Config::getExternalTune()
{
	if ( module != NULL && strlen(module) > 0)
		return module;
	else
		return NULL;
}

void Config::setExternalTune(char* _module)
{
	module = _module;
}


Vector* Config::getMixOptions()
{
	return mixOptions;
}

char* Config::getArticle()
{
	return article;
}

void Config::setArticle(char* _article)
{
	article = _article;
}

char* Config::vectorToString(Vector* vector)
{
	Uint32 length = 0;
	
	/* Number of ';' characters, and the terminating 0*/
	length += Vector_getSize(vector) - 1 + 1;
	
	for (int i = 0; i < Vector_getSize(vector); i++)
	{
		char* next = (char*) Vector_get(vector, i);
		length += strlen(next);
	}
	
	if (length >  0)
	{
		char* result = new char[length];
		result[0] = 0;		
		for (int i = 0; i < Vector_getSize(vector); i++)
		{
			char* next = (char*) Vector_get(vector, i);
			strcat(result, next);
			if (i+1 < Vector_getSize(vector))
				strcat(result, ";");
			
		}
		return result;
	}
	else
		return "";
}

char* Config::getExternalLink()
{
	return externalLink;
}

void Config::setExternalLink(char* _externalLink)
{
	externalLink = _externalLink;
}
