#ifndef SDLConfig_h
#define SDLConfig_h

#include "Map.h"
#include "Vector.h"
#include "SDL/SDL.h"
#include "Platform.h"
#include <stdio.h>

#define SDLCFG_FULLSCREEN "fullscreen"
#define SDLCFG_VERBOSE "verbose"
#define SDLCFG_EFFECTS "effects"
#define SDLCFG_SHOW_CONFIG "showConfig"
#define SDLCFG_MIXING_FREQUENCY "mixingFrequency"
#define SDLCFG_CURRENT_DIR "currentDir"
#define SDLCFG_FONT_PATHS "fontPaths"
#define SDLCFG_MUSIC "music"
#define SDLCFG_SOUND_EFFECTS "soundEffects"
#define SDLCFG_MIX_OPTIONS "mixOptions"
#define SDLCFG_EXTERNAL_LINK "externalLink"

class Config
{
	public:
	
	Config(char* _path, Platform* platform);
	
	bool getFullscreen();
	bool getVerbose();
	bool getEffects();
	bool getShowConfig();
	bool getMusic();
	bool getSoundEffects();
		
	bool* getFullscreenPtr();
	bool* getVerbosePtr();
	bool* getEffectsPtr();
	bool* getShowConfigPtr();
	bool* getMusicPtr();
	bool* getSoundEffectsPtr();

	Uint32 getMixingFrequency();
	Vector* getFontPaths();
	Vector* getMixOptions();
	char* getExternalTune();
	char* getArticle();
	char* getExternalLink();
	
	void setFullscreen(bool _fullscreen);
	void setVerbose(bool _verbose);
	void setEffects(bool _effects);
	void setShowConfig(bool _showConfig);
	void setMixingFrequency(Uint32 _mixingFrequency);
	void setMusic(bool _music);
	void setSoundEffects(bool _soundEffects);
	
	void setExternalTune(char* _module);
	void setArticle(char* _article);
	void setExternalLink(char* _externalLink);
	
	/* Read different datatypes. Return value indicates whetere they keyword could be found */	
	bool readBool(char* keyword, bool* value);
	bool readString(char* keyword, char** value);
	bool readUint(char* keyword, Uint32* value);
	bool readStringCollection(Vector* vector, char* keyword);
	
	/* Write this configuration back out to the path it came from */
	void save();


	void writeBool(FILE* file, char* keyword, bool value);
	void writeInt(FILE* file, char* keyword, int value);
	void writeString(FILE* file, char* keyword, char* value);
	
	private:

	char* vectorToString(Vector* vector);
	
	char* path;
	
	bool fullscreen;
	bool verbose;
	bool effects;
	bool showConfig;
	bool music;
	bool soundEffects;

	Uint32 mixingFrequency;
	Vector* fontPaths;
	Vector* mixOptions;
	char* module;
	char* article;
	char* externalLink;
	
	Map* configMap;
};

#endif
