#ifndef SDLFont_h
#define SDLFont_h

#include "SDL/SDL.h"
#include "Vector.h"
#include <ft2build.h>  
#include FT_FREETYPE_H

typedef struct
{
	Vector* fonts;
} SDLFace;

typedef struct
{
	SDL_Surface* bitmap;
	int xDisplace;
	int yDisplace;
	int advance;
	
} SDLCharacter;

typedef struct
{
	Uint32 ascender;
	Uint32 descender;
	Uint32 colour;
	SDLCharacter* characters[256];
} SDLFont;


/* Load and creates a SDLFace from the filesystem */
SDLFace* SDLFace_load(char* path, Uint32 size);

/* Creates an SDLFace structure, and renderes a plain white version */
SDLFace* SDLFace_create(FT_Face face);

/* Get an instance of the face, which have the right colour. If one does not yet exist, one will be created.*/
SDLFont* SDLFace_getColour( SDLFace* face, Uint32 colour );

/* Return an instance of any colour */
SDLFont* SDLFace_getAny( SDLFace* face);



/* Create a copy of the given font, with the indicated colour */
SDLFont* SDLFont_createFrom( SDLFont* font, Uint32 colour);

/* Will create an SDLFont with the given colours */
SDLFont* SDLFont_create(FT_Face face, Uint8 red, Uint8 green, Uint8 blue);

/* Gets a specific character from a font */
SDLCharacter* SDLFont_getCharacter( SDLFont* font, char character );

/* Calculates the length, in pixels, of a text string, when rendered with the given font */
Uint32 SDLFont_getWidth(SDLFont* font, char* string);

/* Calculates the length, in pixels, of the substring defined by from-to, the to character not included */
Uint32 SDLFont_getWidthSubstring(SDLFont* font, char* string, Uint32 from, Uint32 to);

/* Draw the given text, using the given font on the surface at the given coordinates */
Uint32 SDLFont_draw(SDLFont* font, char* string, Uint32 textLength, SDL_Surface* surface, Uint32 x, Uint32 y);




/* Will copy a character, and change its colour */
SDLCharacter* SDLCharacter_copy( SDLCharacter* character, Uint32 colour);

SDLCharacter* SDLCharacter_create(FT_Face face, char character, Uint8 red, Uint8 green, Uint8 blue);

#endif
