#include "SDL_Draw.h"
#include <math.h>
#include <assert.h>
#include <string.h>
#include <stdlib.h>
#include "../../core/basedefs.h"

Uint32 averageArea(SDL_Surface* surface, double startX, double startY, double endX, double endY);

void SDL_StretchSurface(SDL_Surface* source, SDL_Surface* target, Uint32 x, Uint32 y, Uint32 width, Uint32 height)
{
	Uint8* pixels 		= (Uint8*)target->pixels;
	SDL_PixelFormat* fmt 	= source->format;
	SDL_PixelFormat* fmtdst	= target->format;
	unsigned int i = 0;
	unsigned int j = 0;
	
	double ratioWidth 	= (double) source->w / (double) width;
	double ratioHeight 	= (double) source->h / (double) height;
	
	for (i = 0; i < width; i++)
	{
		for (j = 0; j < height; j++)
		{
			Uint32 color		= 0;
			Uint32 destination	= 0;
			double doubleAlpha 	= 0;

			// Calculate the colour to set here			
			color = averageArea(source, i*ratioWidth, j*ratioHeight, (i+1)*ratioWidth, (j+1)*ratioHeight);
			
			//Read all components of src and dst, since we need to do alpha blending
			Uint8 srcRed 	= (Uint8)( ((color & fmt->Rmask) >> fmt->Rshift) << fmt->Rloss);
			Uint8 srcGreen 	= (Uint8)( ((color & fmt->Gmask) >> fmt->Gshift) << fmt->Gloss);
			Uint8 srcBlue 	= (Uint8)( ((color & fmt->Bmask) >> fmt->Bshift) << fmt->Bloss);
			Uint8 srcAlpha 	= (Uint8)( ((color & fmt->Amask) >> fmt->Ashift) << fmt->Aloss);
			
			destination = *(Uint32*)(pixels+(y+j)*target->pitch + ((x+i)<<2));
			
			Uint8 dstRed 	= (Uint8)( ((destination & fmtdst->Rmask) >> fmtdst->Rshift) << fmtdst->Rloss);
			Uint8 dstGreen 	= (Uint8)( ((destination & fmtdst->Gmask) >> fmtdst->Gshift) << fmtdst->Gloss);
			Uint8 dstBlue 	= (Uint8)( ((destination & fmtdst->Bmask) >> fmtdst->Bshift) << fmtdst->Bloss);
			
			//Calculate final alpha value
			doubleAlpha = ((double)srcAlpha/255.0) * ((double)fmt->alpha/255.0);
			
			//Calculate alpha blended pixel colors
			dstRed 		= (Uint8)(doubleAlpha * srcRed + (1-doubleAlpha) * dstRed);
			dstGreen 	= (Uint8)(doubleAlpha * srcGreen + (1-doubleAlpha) * dstGreen);
			dstBlue 	= (Uint8)(doubleAlpha * srcBlue + (1-doubleAlpha) * dstBlue);
			
			//Create the final color
			color = 0;
			color = ((Uint32)dstRed)<<fmtdst->Rshift | ((Uint32)dstGreen)<<fmtdst->Gshift | ((Uint32)dstBlue)<<fmtdst->Bshift;
			
			*(Uint32*)(pixels+(y+j)*target->pitch + ((x+i)<<2)) = color;
		}
	}
}

Uint32 averageArea(SDL_Surface* surface, double startX, double startY, double endX, double endY)
{
	/* Extract some pointers and value, for readability later on */
	SDL_PixelFormat* fmt 	= surface->format;
	Uint8* pixels 		= ((Uint8*)surface->pixels);
	Uint32 pitch 		= surface->pitch;
	
	/* Determine integer borders */
	Uint32 lowX 	= (Uint32) startX;
	Uint32 lowY 	= (Uint32) startY;
	Uint32 highX 	= (Uint32) ceil(endX-1);
	Uint32 highY 	= (Uint32) ceil(endY-1);

	/* Iteration indices */	
	Uint32 x = 0;
	Uint32 y = 0;

	/* Total size of area */
	double totalArea = (endX-startX)*(endY-startY);
	
	/* Will hold the components of the colours */
	double red 	= 0;
	double green 	= 0;
	double blue 	= 0;
	double alpha	= 0;

	for (x = lowX; x <= highX; x++)
	{
		for (y = lowY; y <= highY; y++)
		{
			/* The area of this pixel */
			double pixelArea = 1;
			
			/* Extract the colour components of this pixel */
			Uint32 pixel = *(Uint32*)(pixels + y*pitch + (x * fmt->BytesPerPixel) );
			
			Uint8 redElement 	= (Uint8)( ((pixel & fmt->Rmask) >> fmt->Rshift) << fmt->Rloss);
			Uint8 greenElement 	= (Uint8)( ((pixel & fmt->Gmask) >> fmt->Gshift) << fmt->Gloss);
			Uint8 blueElement 	= (Uint8)( ((pixel & fmt->Bmask) >> fmt->Bshift) << fmt->Bloss);
			Uint8 alphaElement 	= (Uint8)( ((pixel & fmt->Amask) >> fmt->Ashift) << fmt->Aloss);
			
			double pixStartX = 0;
			double pixStartY = 0;
			double pixEndX = 1;
			double pixEndY = 1;

			/* Determine how much of this pixel is to be included */
			if (x == lowX)
				pixStartX = startX-(double)lowX;
			if (x == highX)
				pixEndX = 1.0-((double)highX+1-endX);
			if (y == lowY)
				pixStartY = startY-(double)lowY;
			if (y == highY)
				pixEndY = 1.0-((double)highY+1-endY);
			
			pixelArea = (pixEndX-pixStartX)*(pixEndY-pixStartY);
			
			red 	+= ((double)redElement) * pixelArea / totalArea;
			green 	+= ((double)greenElement) * pixelArea / totalArea;
			blue 	+= ((double)blueElement) * pixelArea / totalArea;
			alpha 	+= ((double)alphaElement) * pixelArea / totalArea;
		}
	}
	
	return ((Uint32)red)<<fmt->Rshift | ((Uint32)green)<<fmt->Gshift | ((Uint32)blue)<<fmt->Bshift | ((Uint32)alpha)<<fmt->Ashift;
}

short* deltaValues = NULL;

/* Update fadeFrom with crossfade to become equal to fadeTo */
void SDL_Crossfade(SDL_Surface *fadeTo, SDL_Surface *fadeFrom, Uint32 duration)
{
	assert (fadeTo->w == fadeFrom->w && fadeTo->h == fadeFrom->h);
	assert (duration >= 0);
	assert (fadeTo != NULL);
	assert (fadeFrom != NULL);

	int width = fadeTo->w;
	int height = fadeTo->h;
	
	/* A short should be enough, but int seems to be faster - funny */
        if (deltaValues == NULL)
	       deltaValues = (short*) malloc ( width * height * 3 * sizeof(short) );	
	
	for (int i = 0; i < width; i++)
	{
		for (int j = 0; j < height; j++)
		{
			Uint32 srcColour = SDL_getColor(fadeFrom, i, j);
			Uint32 dstColour = SDL_getColor(fadeTo, i, j);
			Uint8 srcRed, srcGreen, srcBlue, dstRed, dstGreen, dstBlue;
			
			SDL_GetRGB(srcColour, fadeFrom->format, &srcRed, &srcGreen, &srcBlue);
			SDL_GetRGB(dstColour, fadeTo->format, &dstRed, &dstGreen, &dstBlue);
	
			deltaValues[j*width*3 + i*3] 		= (short)srcRed-(short)dstRed; 
			deltaValues[j*width*3 + i*3 + 1] 	= (short)srcGreen-(short)dstGreen;
			deltaValues[j*width*3 + i*3 + 2] 	= (short)srcBlue-(short)dstBlue;
		}
	}
	
	Uint32 startTime 	= SDL_GetTicks();
	Uint32 currentTime 	= startTime;
	Uint32 elapsedTime	= 0;
	float timePercent	= 0;

	Uint32 offset = 0;

	currentTime = SDL_GetTicks();
	
	
	int redOffset = 0;
	int greenOffset = 0;
	int blueOffset = 0;
	SDL_PixelFormat* format = fadeFrom->format;
	Uint32 Rmask = format->Rmask;
	Uint32 Gmask = format->Gmask;
	Uint32 Bmask = format->Bmask;
	Uint32 initMask = 0x000000ff;
#if __BYTE_ORDER == __BIG_ENDIAN
		initMask = 0xff000000;
#endif
	for (int i = 0; i < 4; i++)
	{
		if (Rmask == initMask)
			redOffset = i;
		else if (Gmask == initMask)
			greenOffset = i;
		else if (Bmask == initMask)
			blueOffset = i;
#if __BYTE_ORDER == __BIG_ENDIAN
	initMask = initMask >> 8;
#else
	initMask = initMask << 8;
#endif
	}

	/* Adjust fadeTo.alpha, and use SDL_Blit to do the crossfade */
	bool continueFade = true;
	Uint8 BytesPerPixel = fadeTo->format->BytesPerPixel;
	float inverseTimePercent = 0;
	Uint8* src = NULL;
	Uint8* dst = NULL;
	while (continueFade)
	{
		elapsedTime = currentTime - startTime;
		
		if (elapsedTime >= duration)
		{
			elapsedTime = duration;
			continueFade = false;
		}
		timePercent = (double)elapsedTime / (double)duration;
		inverseTimePercent = 1 - timePercent; //Pre calc cause I need 1-timepercent in loop
		
		int timeAdjust = (int)(inverseTimePercent*256.0);
		
		Uint8* lineTo 		= (Uint8*) fadeTo->pixels;
		Uint8* lineFrom 	= (Uint8*) fadeFrom->pixels;
		offset 			= 0;
		
		for (int j = 0; j < height; j++)
		{
			src = lineFrom;
			dst = lineTo;
			
			for (int i = 0; i < width; i++)
			{
				src[redOffset] 	= dst[redOffset] + (deltaValues[offset++]*timeAdjust>>8);
				src[greenOffset] 	= dst[greenOffset] + (deltaValues[offset++]*timeAdjust>>8);
				src[blueOffset] 	= dst[blueOffset] + (deltaValues[offset++]*timeAdjust>>8);

				src 	+= BytesPerPixel;
				dst	+= BytesPerPixel;
			}
			lineTo 		+= fadeTo->pitch;	
			lineFrom 	+= fadeFrom->pitch;
		}
		
		SDL_UpdateRect(fadeFrom, 0, 0, 0, 0);

		currentTime 	= SDL_GetTicks();
	}
}

inline Uint32* SDL_getLine(SDL_Surface* surface, Uint32 y)
{
	return (Uint32*)( ((Uint8*)surface->pixels) + y*surface->pitch );
}

inline Uint32 SDL_getColor(SDL_Surface* surface, Uint32 x, Uint32 y)
{
	return *(Uint32*)( ((Uint8*)surface->pixels) + y*surface->pitch + (x * surface->format->BytesPerPixel) );
}

inline void SDL_setColor(SDL_Surface* surface, Uint32 x, Uint32 y, Uint32 colour)
{
	*(Uint32*)( ((Uint8*)surface->pixels) + y*surface->pitch + (x * surface->format->BytesPerPixel) ) = colour;
}
